##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

"""Presentation Template Expression Engine

Presentation Template-specific implementation of TALES, with handlers
for Python expressions, Python string literals, and paths.
"""

__version__='$Revision: 1.4 $'[11:-2]

import ZODB
from TALES import Engine, TALESError, _valid_name
from string import strip, split, join
import re
from AccessControl import getSecurityManager


_engine = None
def getEngine():
    global _engine
    if _engine is None:
        _engine = Engine()
        installHandlers(_engine)
    return _engine

def installHandlers(engine):
    reg = engine.registerType
    pe = PathExpr
    for pt in ('standard', 'path', 'var', 'local', 'global', 'loop'):
        reg(pt, pe)
    reg('str', StrExpr)
    reg('python', PythonExpr)
    reg('not', NotExpr)
    reg('import', ImportExpr)

class PathExpr:
    def __init__(self, name, expr):
        if name in ('standard', 'path'): name = None
        self._baseContext = name
        self._path = path = split(expr, '/')
        self._dynparts = dp = []
        for i in range(len(path)):
            e = path[i]
            if e[:1] == '?' and _valid_name(e[1:]):
                dp.append((i, e[1:]))
        dp.reverse()
    def __call__(self, econtext):
        base = self._baseContext
        path = self._path
        contexts = econtext.contexts
        var = contexts['var']
        # Expand dynamic path parts from right to left.
        for i, varname in self._dynparts:
            val = var[varname]
            if type(val) is type(''):
                path[i] = val
            else:
                # If the value isn't a string, assume it's a sequence
                # of path names.
                path[i:i+1] = list(val)

        if base is None:
            # This is a standard path, so pick a starting object.
            if path[0] == '':
                # Absolute path can traverse from any acquirer.
                ob = contexts['container']
            else:
                # Relative path must start with a context name.
                ob = contexts[path.pop(0)]
        else:
            ob = contexts[base][path.pop(0)]
        ob = restrictedTraverse(ob, path)
        # XXX we need some way to spell nocall semantics
        if callable(ob):
            try: return ob()
            except AttributeError, v:
                if v[0] != '__call__': raise

        return ob
            

class StrExpr:
    def __init__(self, name, expr):
        self._s = expr
    def __call__(self, econtext):
        return self._s % econtext.contexts['var']
    def __repr__(self):
        return '<StrExpr %s>' % `self._s`

class ImportExpr:
    def __init__(self, name, expr):
        self._s = expr
    def __call__(self, econtext):
        _silly=('__doc__',)
        _globals={}
        module=__import__(self._s, _globals, _globals, _silly)
        return module
    def __repr__(self):
        return '<ImportExpr %s>' % `self._s`

class NotExpr:
    def __init__(self, name, expr, compiler):
        self._s = expr
        self._c = compiler.compile(expr)
        
    def __call__(self, econtext):
        return not self._c(econtext)

    def __repr__(self):
        return '<NotExpr %s>' % `self._s`
    

from Products.PythonScripts.Guarded import \
     GuardedBlock, theGuard, safebin, WriteGuard, ReadGuard, UntupleFunction

class PythonExpr:
    def __init__(self, name, expr):
        blk = GuardedBlock('def f():\n return %s\n' % strip(expr))
        if blk.errors:
            raise TALESError, ('Python expression error:\n%s' %
                                          join(blk.errors, '\n') )
        guards = {'$guard': theGuard, '$write_guard': WriteGuard,
                  '$read_guard': ReadGuard, '__debug__': __debug__}
        self._f = UntupleFunction(blk.t, guards, __builtins__=safebin)
    def __call__(self, econtext):
        f = self._f
        f.func_globals.update(econtext.contexts)
        f.func_globals['contexts'] = econtext.contexts
        # Execute the function in a new security context.
        security = getSecurityManager()
        security.addContext(self)
        try:
            return f()
        finally:
            security.removeContext(self)

def restrictedTraverse(self, path):

    if not path: return self

    get=getattr
    N=None
    M=[] #marker

    REQUEST={'TraversalRequestNameStack': path}
    path.reverse()
    pop=path.pop
    securityManager=getSecurityManager()

    if not path[-1]:
        # If the path starts with an empty string, go to the root first.
        pop()
        self=self.getPhysicalRoot()
        #if not securityManager.validateValue(self):
        #    raise 'Unauthorized', name
                    
    object = self
    while path:
        name=pop()

        if name[0] == '_':
            # Never allowed in a URL.
            raise 'NotFound', name

        if name=='..':
            o=getattr(object, 'aq_parent', M)
            if o is not M:
                #if not securityManager.validate(object, object, name, o):
                #    raise 'Unauthorized', name
                object=o
                continue

        t=get(object, '__bobo_traverse__', N)
        if t is not N:
            o=t(REQUEST, name)
                    
            # Note we pass no container, because we have no
            # way of knowing what it is
            #if not securityManager.validate(object, None, name, o):
            #    raise 'Unauthorized', name
                      
        else:
            o=get(object, name, M)
            if o is not M:
                # waaaa
                if hasattr(get(object,'aq_base',object), name):
                    # value wasn't acquired
                    #if not securityManager.validate(
                    #    object, object, name, o):
                    #    raise 'Unauthorized', name
                    pass
                else:
                    #if not securityManager.validate(
                    #    object, None, name, o):
                    #    raise 'Unauthorized', name
                    pass
                        
            else:
                o=object[name]
                #if not securityManager.validate(object, object, None, o):
                #    raise 'Unauthorized', name
        object = o

    return object
