##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
"""Presentation Templates Product

Provide support for XHTML-based template objects.
"""

__version__='$Revision: 1.8 $'[11:-2]

from Globals import DTMLFile, MessageDialog
from urllib import quote
from TAL.TALParser import TALParser
from TAL.TALGenerator import TALGenerator
from TAL.TALInterpreter import TALInterpreter
from Expressions import getEngine
from zLOG import LOG, ERROR, INFO
import AccessControl, Acquisition
from OFS.SimpleItem import SimpleItem
from DateTime.DateTime import DateTime
from string import join, strip, rstrip, split, replace, lower
from Shared.DC.Scripts.Script import Script, BindingsUI
from Shared.DC.Scripts.Signature import FuncCode
from AccessControl import getSecurityManager
from OFS.History import Historical, html_diff
from OFS.Cache import Cacheable
from StringIO import StringIO # XXX Not from cStringIO?

defaultBindings = {'name_context': 'here',
                   'name_container': 'container',
                   'name_m_self': 'template',
                   'name_ns': '',
                   'name_subpath': 'traverse_subpath'}

class MacroCollection(SimpleItem):
    def __of__(self, parent):
        parent._cook()
        pr, ma = parent._v_cooked
        return ma

default_src="""<?xml version="1.0" ?>
<html xmlns:tal="http://xml.zope.org/namespaces/tal"
      xmlns:metal="http://xml.zope.org/namespaces/metal">
  <head>
    <title tal:insert="here/title">The title</title>
  </head>
  <body>
  </body>
</html>
"""

def manage_addPresentationTemplate(self, id, REQUEST=None, submit=None):
    "Add a Presentation Template with optional file content."

    id = str(id)
    id = self._setObject(id, PresentationTemplate(id))
    if REQUEST is not None:
        file = REQUEST.form.get('file', '')
        content_type='text/html'
        if type(file) is not type(''):
            headers=getattr(file, 'headers', None)
            if headers and headers.has_key('content-type'):
                content_type=headers['content-type']
            file = file.read()
            
        if file:
            self._getOb(id).write(file)
        else:
            self._getOb(id).write(default_src)

        self._getOb(id).content_type=content_type
            
        try: u = self.DestinationURL()
        except: u = REQUEST['URL1']
        if submit==" Add and Edit ": u="%s/%s" % (u,quote(id))
        REQUEST.RESPONSE.redirect(u+'/manage_main')
    return ''


manage_addPresentationTemplateForm = DTMLFile('dtml/ptAdd', globals())

class PresentationTemplate(Script, Historical, Cacheable):
    "Presentation Templates using TAL, TALES, and METAL"
     
    meta_type = 'Presentation Template'

    content_type='text/html'

    _v_cooked = 0

    _text = ''
    func_defaults = None
    func_code = FuncCode((), 0)

    macros = MacroCollection()

    manage_options = (
        {'label':'Edit', 'action':'pt_editForm'},
        {'label':'Test', 'action':'ZScriptHTML_tryForm'},
        ) + Historical.manage_options + SimpleItem.manage_options + \
        Cacheable.manage_options

    def __init__(self, id):
        self.id = id
        self.ZBindings_edit(defaultBindings)

    security = AccessControl.ClassSecurityInfo()

    security.declareObjectProtected('View')
    security.declareProtected('View', '__call__')

    security.declareProtected('View management screens',
      'pt_editForm', 'manage_main', 'read',
      'ZScriptHTML_tryForm', 'PrincipiaSearchSource',
      'document_src')

    pt_editForm = DTMLFile('dtml/ptEdit', globals())
    manage = manage_main = pt_editForm

    security.declareProtected('Change Python Scripts',
      'pt_editAction', 'pt_setTitle', 'pt_edit',
      'pt_upload', 'pt_changePrefs')
    def pt_editAction(self, REQUEST, title, text, content_type):
        """Change the title and document."""
        self.pt_setTitle(title)
        self.pt_edit(text, content_type)
        message = "Saved changes."
        if getattr(self, '_v_warnings', None):
            message = ("<strong>Warning:</strong> <i>%s</i>" 
                       % join(self._v_warnings, '<br>'))
        return self.pt_editForm(self, REQUEST, manage_tabs_message=message)

    def pt_setTitle(self, title):
        title = str(title)
        if self.title != title:
            self.title = title
            self.ZCacheable_invalidate()

    def pt_edit(self, text, content_type):
        if type(text) is not type(''):
            text = text.read()
        if self._text <> text:
            self.write(text)
        if content_type and content_type != self.content_type:
            self.content_type=content_type

    def pt_upload(self, REQUEST, file=''):
        """Replace the document with the text in file."""
        if type(file) is not type(''): file = file.read()
        self.write(file)
        message = 'Saved changes.'
        return self.pt_editForm(self, REQUEST, manage_tabs_message=message)

    def pt_changePrefs(self, REQUEST, height=None, width=None,
                       dtpref_cols='50', dtpref_rows='20'):
        """Change editing preferences."""
        szchh = {'Taller': 1, 'Shorter': -1, None: 0}
        szchw = {'Wider': 5, 'Narrower': -5, None: 0}
        try: rows = int(height)
        except: rows = max(1, int(dtpref_rows) + szchh.get(height, 0))
        try: cols = int(width)
        except: cols = max(40, int(dtpref_cols) + szchw.get(width, 0))
        e = (DateTime('GMT') + 365).rfc822()
        setc = REQUEST['RESPONSE'].setCookie
        setc('dtpref_rows', str(rows), path='/', expires=e)
        setc('dtpref_cols', str(cols), path='/', expires=e)
        REQUEST.form.update({'dtpref_cols': cols, 'dtpref_rows': rows})
        return apply(self.manage_main, (self, REQUEST), REQUEST.form)

    def ZScriptHTML_tryParams(self):
        """Parameters to test the script with."""
        return []

    def manage_historyCompare(self, rev1, rev2, REQUEST,
                              historyComparisonResults=''):
        return PresentationTemplate.inheritedAttribute(
            'manage_historyCompare')(
            self, rev1, rev2, REQUEST,
            historyComparisonResults=html_diff(rev1.read(), rev2.read()) )

    def _exec(self, bound_names, args, kw):
        """Call a PresentationTemplate"""
        # Prepare the template
        self._cook()
        pr, ma = self._v_cooked
        bound_names['nothing'] = None
        bound_names['keywords'] = kw
        ctxt = apply(getEngine().getContext, (), bound_names)

        __traceback_info__ = bound_names

        # Execute the template in a new security context.
        security=getSecurityManager()
        security.addContext(self)
        try:
            out = StringIO()
            TALInterpreter(pr, ma, ctxt, out,
                           html=self.content_type=='text/html',
                           )()
            try: self.REQUEST.RESPONSE.setHeader('content-type',
                                                 self.content_type)
            except AttributeError: pass
            return out.getvalue()
        finally:
            security.removeContext(self)

    security.declareProtected('Change Presentation Templates',
      'PUT', 'manage_FTPput', 'write',
      'manage_historyCopy',
      'manage_beforeHistoryCopy', 'manage_afterHistoryCopy')

    def PUT(self, REQUEST, RESPONSE):
        """ Handle HTTP PUT requests """
        self.dav__init(REQUEST, RESPONSE)
        self.write(REQUEST.get('BODY', ''))
        RESPONSE.setStatus(204)
        return RESPONSE        

    manage_FTPput = PUT

    def write(self, text):
        self._v_cooked = 0
        self._text = text 

    def manage_FTPget(self):
        "Get source for FTP download"
        self.REQUEST.RESPONSE.setHeader('Content-Type', self.content_type)
        return self.read()

    def last_error(self):
        try: self._cook()
        except: pass
        return self._v_cookerr

    def read(self):
        try: self._cook()
        except: return self._text
        pr, ma = self._v_cooked
        ctxt = getEngine().getContext(template=self,
                                      here=self.aq_parent,
                                      container=self.aq_inner.aq_parent)
        out = StringIO()
        TALInterpreter(pr, ma, ctxt, out, tal=0,
                       html=self.content_type=='text/html')()
        return out.getvalue()

    def get_size(self): return len(self._text)
    getSize = get_size

    def PrincipiaSearchSource(self):
        "Support for searching - the document's contents are searched."
        return self.read()

    def document_src(self, REQUEST=None, RESPONSE=None):
        """Return expanded document source."""

        if RESPONSE is not None:
            RESPONSE.setHeader('Content-Type', self.content_type)
        return self.read()

    def _cook(self):
        if self._v_cooked: return
        try: 
            engine = None # XXX why not getEngine()?
            gen = TALGenerator(engine)
            parser = TALParser(gen)
            parser.parseString(self._text)
            self._v_cooked = parser.getCode()
        except Exception, v:
            self._v_cookerr=str(v[0])
            raise
        else:
            self._v_cookerr=''
            

    def html(self): return self.content_type=='text/html'

class Src(Acquisition.Explicit):
    " "

    PUT=document_src=Acquisition.Acquired
    index_html=None
    
    def __call__(self, REQUEST, RESPONSE):
        " "
        return self.document_src(REQUEST, RESPONSE)

d=PresentationTemplate.__dict__
d['source.xml']=d['source.html']=Src()
