##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

import sys, util

import Zope # Waaaa
import Products.PresentationTemplates.PresentationTemplate

class DTMLTests(util.unittest.TestCase):

   def setUp(self):
      self.t=(Products.PresentationTemplates.PresentationTemplate.
              PresentationTemplate('test id'))


   def check1(self):
      """DTML test 1: if, in, and var:

      %(comment)[ blah %(comment)]
      <html><head><title>Test of documentation templates</title></head>
      <body>
      %(if args)[
      <dl><dt>The arguments to this test program were:<p>
      <dd>
      <ul>
      %(in args)[
        <li>Argument number %(num)d was %(arg)s
      %(in args)]
      </ul></dl><p>
      %(if args)]
      %(else args)[
      No arguments were given.<p>
      %(else args)]
      And thats da trooth.
      </body></html>
      """

      tal='''
        <html xmlns:tal="http://xml.zope.org/namespaces/tal">
        <head><title>Test of documentation templates</title></head>
        <body>
        <span tal:replace="nothing"> blah </span>
        <dl tal:condition="here/args">
          <dt>The arguments to this test program were:</dt>
          <dd>
          <ul>
            <li tal:repeat="arg here/args">
            Argument number <span tal:replace="local:arg/num">99</span>
            is <span tal:replace="local:arg/arg">default</span>
            </li>
          </ul></dd></dl>
        <p tal:condition="not:here/args">No arguments were given.</p>
        And thats da trooth.
        </body></html>
        '''

      self.t.write(tal)

      aa=util.argv(('one', 'two', 'three', 'cha', 'cha', 'cha'))
      o=self.t.__of__(aa)()
      expect=('''<html xmlns:tal="http://xml.zope.org/namespaces/tal">
            <head><title>Test of documentation templates</title></head>
            <body>
            
            <dl tal:condition="here/args">
              <dt>The arguments to this test program were:</dt>
              <dd>
              <ul>
                <li tal:repeat="arg here/args">
                Argument number 1
                is one
                </li><li tal:repeat="arg here/args">
                Argument number 2
                is two
                </li><li tal:repeat="arg here/args">
                Argument number 3
                is three
                </li><li tal:repeat="arg here/args">
                Argument number 4
                is cha
                </li><li tal:repeat="arg here/args">
                Argument number 5
                is cha
                </li><li tal:repeat="arg here/args">
                Argument number 6
                is cha
                </li>
              </ul></dd></dl>
            
            And thats da trooth.
            </body></html>
            ''')            
      util.check_xml(expect, o)

      aa=util.argv(())
      o=self.t.__of__(aa)()
      expect='''<html xmlns:tal="http://xml.zope.org/namespaces/tal">
        <head><title>Test of documentation templates</title></head>
        <body>
        
        
        <p tal:condition="not:here/args">No arguments were given.</p>
        And thats da trooth.
        </body></html>'''
      util.check_xml(expect, o)

   def check3(self):
      """DTML test 3: batches and formatting:

        <html><head><title>Test of documentation templates</title></head>
        <body>
        <!--#if args-->
          The arguments were:
          <!--#in args size=size end=end-->
              <!--#if previous-sequence-->
                 (<!--#var previous-sequence-start-arg-->-
                  <!--#var previous-sequence-end-arg-->)
              <!--#/if previous-sequence-->
              <!--#if sequence-start-->
                 <dl>
              <!--#/if sequence-start-->
              <dt><!--#var sequence-arg-->.</dt>
              <dd>Argument <!--#var num fmt=d--> was <!--#var arg--></dd>
              <!--#if next-sequence-->
                 (<!--#var next-sequence-start-arg-->-
                  <!--#var next-sequence-end-arg-->)
              <!--#/if next-sequence-->
          <!--#/in args-->
          </dl>
        <!--#else args-->
          No arguments were given.<p>
        <!--#/if args-->
        And I\'m 100% sure!
        </body></html>
      """

      tal=('''\
        <html xmlns:tal="http://xml.zope.org/namespaces/tal">
        <head><title>Test of documentation templates</title></head>
        <body>
        <p tal:condition="here/args">
          The arguments were:
          <span tal:condition="keywords/batch/previous_sequence">
            (<span
              tal:replace="keywords/batch/previous_sequence_start_item"
              >previous start item</span>-<span
              tal:replace="keywords/batch/previous_sequence_end_item"
              >previous end item</span>)
          </span>
          <dl>
            <span tal:repeat="arg keywords/batch">
              <dt><span tal:replace="local/arg">??</span>.</dt>
              <dd>Argument <span tal:define="num local:arg/num"
                                 tal:replace="str: %(num)d"
                  >99</span> was <span tal:replace="local:arg"
                  >??</span></dd>
            </span>
          </dl>
          <span tal:condition="keywords/batch/next_sequence">
            (<span
              tal:replace="keywords/batch/next_sequence_start_item"
              >next start item</span>-<span
              tal:replace="keywords/batch/next_sequence_end_item"
              >next end item</span>)
          </span>
        </p>
        <p tal:condition="not:here/args">
          No arguments were given.
        </p>
        And I am 100% sure!
        </body></html>
        ''')

      self.t.write(tal)

      aa=util.argv(('one', 'two', 'three', 'four', 'five',
                    'six', 'seven', 'eight', 'nine', 'ten',
                    'eleven', 'twelve', 'thirteen', 'fourteen', 'fifteen',
                    'sixteen', 'seventeen', 'eighteen', 'nineteen', 'twenty',
                    ))
      import batch        
      o=self.t.__of__(aa)(batch=batch.batch(aa.args, 5))

      expect='''<html xmlns:tal="http://xml.zope.org/namespaces/tal">
        <head><title>Test of documentation templates</title></head>
        <body>
        <p tal:condition="here/args">
          The arguments were:
          
          <dl>
            <span tal:repeat="arg keywords/batch">
              <dt>one.</dt>
              <dd>Argument  1 was one</dd>
            </span><span tal:repeat="arg keywords/batch">
              <dt>two.</dt>
              <dd>Argument  2 was two</dd>
            </span><span tal:repeat="arg keywords/batch">
              <dt>three.</dt>
              <dd>Argument  3 was three</dd>
            </span><span tal:repeat="arg keywords/batch">
              <dt>four.</dt>
              <dd>Argument  4 was four</dd>
            </span><span tal:repeat="arg keywords/batch">
              <dt>five.</dt>
              <dd>Argument  5 was five</dd>
            </span>
          </dl>
          <span tal:condition="keywords/batch/next_sequence">
            (six-ten)
          </span>
        </p>
        
        And I am 100% sure!
        </body></html>
      '''
      
      util.check_xml(expect, o)

def test_suite():
   return util.unittest.makeSuite(DTMLTests, 'check')

def main():
   util.unittest.TextTestRunner().run(test_suite())

def debug():
   test_suite().debug()
   
if __name__=='__main__':
   if len(sys.argv) > 1:
      globals()[sys.argv[1]]()
   else:
      main()

